@@ -2,21 +2,21 @@ use crate::MiniIter;
 
 /// Tokens are the intermediate representation format in the markdown to html conversion
 #[derive(Debug, PartialEq)]
-pub enum Token {
+pub enum Token<'a> {
     /// String: Body of unstructured text
     Plaintext(String),
-    /// u8: Header level (1..=6). String: Header text. Option<String>: html label
-    Header(u8, String, Option<String>),
-    /// String: Text for list entry
-    UnorderedListEntry(String),
-    /// String: Text for list entry
-    OrderedListEntry(String),
-    /// String: Text to be italicized
-    Italic(String),
-    /// String: Text to be bolded
-    Bold(String),
-    /// String: Text to be bolded and italicized
-    BoldItalic(String),
+    /// u8: Header level (1..=6). str: Header text. Option<str>: html label
+    Header(u8, &'a str, Option<&'a str>),
+    /// str: Text for list entry
+    UnorderedListEntry(&'a str),
+    /// str: Text for list entry
+    OrderedListEntry(&'a str),
+    /// str: Text to be italicized
+    Italic(&'a str),
+    /// str: Text to be bolded
+    Bold(&'a str),
+    /// str: Text to be bolded and italicized
+    BoldItalic(&'a str),
     /// Corresponds to a </br> html tag
     LineBreak,
     /// Corresponds to a newline character
@@ -27,26 +27,26 @@ pub enum Token {
     Tab,
     /// Used for control flow. Not directly rendered
     DoubleTab,
-    /// String: Text to be struck through
-    Strikethrough(String),
-    /// String: Text to be placed within an inline code tag. eg. <code>String</code>
-    Code(String),
-    /// First String: Text to be placed within a multi-line code tag. Second String: Language
-    CodeBlock(String, String),
-    /// u8: Block quote level. String: Block quote text
-    BlockQuote(u8, String),
-    /// String: Link. Option<String>: Title for link.
-    Image(String, Option<String>),
-    /// String: Link. First Option<String>: Title for link. Second Option<String>: Hover text
-    Link(String, Option<String>, Option<String>),
-    /// String: Summary. Vec<Token>: Tokens to be rendered in the collapsable section
-    Detail(String, Vec<Token>),
-    /// Tuple of Vec<(Alignment, String)>: Which defines the table header and Vec<Vec<(Alignment, Vec<Token>)>> which defines the rows
-    Table(Vec<(Alignment, String)>, Vec<Vec<(Alignment, Vec<Token>)>>),
-    /// TaskBox: Boolean state of the checked or unchecked box. String: List item text
-    TaskListItem(TaskBox, String),
-    /// First String: Reference id. Second String: Reference text
-    Footnote(String, String),
+    /// str: Text to be struck through
+    Strikethrough(&'a str),
+    /// str: Text to be placed within an inline code tag. eg. <code>str</code>
+    Code(&'a str),
+    /// First str: Text to be placed within a multi-line code tag. Second str: Language
+    CodeBlock(&'a str, &'a str),
+    /// u8: Block quote level. str: Block quote text
+    BlockQuote(u8, &'a str),
+    /// str: Link. Option<str>: Title for link.
+    Image(&'a str, Option<&'a str>),
+    /// str: Link. First Option<str>: Title for link. Second Option<str>: Hover text
+    Link(&'a str, Option<&'a str>, Option<&'a str>),
+    /// str: Summary. Vec<Token>: Tokens to be rendered in the collapsable section
+    Detail(&'a str, Vec<Token<'a>>),
+    /// Tuple of Vec<(Alignment, str)>: Which defines the table header and Vec<Vec<(Alignment, Vec<Token>)>> which defines the rows
+    Table(Vec<(Alignment, &'a str)>, Vec<Vec<(Alignment, Vec<Token<'a>>)>>),
+    /// TaskBox: Boolean state of the checked or unchecked box. str: List item text
+    TaskListItem(TaskBox, &'a str),
+    /// First str: Reference id. Second str: Reference text
+    Footnote(&'a str, &'a str),
 }
 
 /// Holds the possible states of a taskbox in a task list
@@ -56,7 +56,7 @@ pub enum TaskBox {
     Unchecked,
 }
 
-impl Token {
+impl <'a> Token<'a> {
     fn is_usable_in_table(&self) -> bool {
         match self {
             Token::Code(_) => true,
@@ -107,30 +107,31 @@ pub(crate) fn push_str<'a>(t: &mut Vec<Token>, s: &'a str) {
     }
 }
 
-pub(crate) fn lex_heading<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_heading<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let hashes = char_iter.consume_while_case_holds(&|c| c == "#").unwrap_or("");
     if char_iter.next_if_eq(&" ").is_none(){
         return Err(ParseError{content: hashes});
     }
     let level = std::cmp::min(6, hashes.len() as u8);
-    let mut line = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("").to_string();
+    let line = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
     if line.contains("{#") && 
         line.contains('}') {
-            let heading = line.chars().take_while(|c| c != &'{').collect::<String>();
-            line.replace_range(..heading.len(), "");
-            line.retain(|c| c != '{' && c != '#' && c != '}');
-            return Ok(Token::Header(level, heading.trim().to_string(), Some(line)));
+            let (heading, _title) = line.split_once("{").unwrap_or(("",""));
+            let line = line.strip_prefix(&heading).unwrap_or("");
+            let line = line.strip_prefix("{#").unwrap_or("");
+            let line = line.strip_suffix("}").unwrap_or("");
+            return Ok(Token::Header(level, heading.trim(), Some(line)));
         }
     return Ok(Token::Header(level, line, None));
 }
 
-pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let asterunds = char_iter.consume_while_case_holds(&|c| c == "*" || c == "_").unwrap_or("");
     if asterunds.len() == 1 && char_iter.next_if_eq(&" ").is_some(){
         let s = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
         char_iter.next();
-        return Ok(Token::UnorderedListEntry(s.to_string()))
+        return Ok(Token::UnorderedListEntry(s))
     }
     if asterunds.chars().all(|x| x == '*') && char_iter.peek() == Some(&"\n"){
         return Ok(Token::HorizontalRule)
@@ -140,7 +141,7 @@ pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Resul
             let s = char_iter.consume_while_case_holds(&|c| c != "*" && c != "_").unwrap_or("");
             if char_iter.peek() != Some("*") || char_iter.peek() != Some(&"_"){
                 char_iter.next();
-                return Ok(Token::Italic(s.to_string()))
+                return Ok(Token::Italic(s))
             } else {
                 return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")});
             }
@@ -149,7 +150,7 @@ pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Resul
             let s = char_iter.consume_while_case_holds(&|c| c != "*" && c != "_").unwrap_or("");
             let trailing_astunds = char_iter.consume_while_case_holds(&|c| c == "*" || c == "_").unwrap_or("");
             if trailing_astunds.len() == 2 {
-                return Ok(Token::Bold(s.to_string()))
+                return Ok(Token::Bold(s))
             } else {
                 return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")});
             }
@@ -158,7 +159,7 @@ pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Resul
             let s = char_iter.consume_while_case_holds(&|c| c != "*" && c != "_").unwrap_or("");
             let trailing_astunds = char_iter.consume_while_case_holds(&|c| c == "*" || c == "_").unwrap_or("");
             if trailing_astunds.len() == 3 {
-                return Ok(Token::BoldItalic(s.to_string()))
+                return Ok(Token::BoldItalic(s))
             } else {
                 return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")});
             }
@@ -173,7 +174,7 @@ pub(crate) fn lex_asterisk_underscore<'a>(char_iter: &mut MiniIter<'a>) -> Resul
     }
 }
 
-pub(crate) fn lex_spaces<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>>{
+pub(crate) fn lex_spaces<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>>{
     let spaces = char_iter.consume_while_case_holds(&|c| c == " ").unwrap_or("");
     // Case 1: space in text => return char
     if spaces.len() == 1 {
@@ -192,7 +193,7 @@ pub(crate) fn lex_spaces<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Pars
     Err(ParseError{content: spaces})
 }
 
-pub(crate) fn lex_backticks<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_backticks<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let leading_ticks = char_iter.consume_while_case_holds(&|c| c == "`").unwrap_or("");
     let mut lang = "plaintext";
@@ -205,7 +206,7 @@ pub(crate) fn lex_backticks<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, P
         if leading_ticks.len() != trailing_ticks.len() {
             return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")}) 
         } else {
-            return Ok(Token::Code(s.to_string()))
+            return Ok(Token::Code(s))
         }
     }
     // leading_ticks.len() == 3. Check for lang
@@ -220,11 +221,11 @@ pub(crate) fn lex_backticks<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, P
     if leading_ticks.len() != trailing_ticks.len() {
         return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")}) 
     } else {
-        return Ok(Token::CodeBlock(s.to_string(), lang.to_string()))
+        return Ok(Token::CodeBlock(s, lang))
     }
 }
 
-pub(crate) fn lex_newlines<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_newlines<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let newlines = char_iter.consume_while_case_holds(&|c| c == "\n").unwrap_or("");
     match newlines.len() {
         0..=1 => return Err(ParseError{content: newlines}),
@@ -232,7 +233,7 @@ pub(crate) fn lex_newlines<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Pa
     }
 }
 
-pub(crate) fn lex_blockquotes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_blockquotes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let right_arrows = char_iter.consume_while_case_holds(&|c| c == ">").unwrap_or("");
     match char_iter.peek() {
         Some(" ") => {char_iter.next();},
@@ -240,10 +241,10 @@ pub(crate) fn lex_blockquotes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token,
     }
     let s = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
     char_iter.next_if_eq(&"\n");
-    Ok(Token::BlockQuote(right_arrows.len() as u8, s.to_string()))
+    Ok(Token::BlockQuote(right_arrows.len() as u8, s))
 }
 
-pub(crate) fn lex_images<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_images<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     if char_iter.peek() != Some(&"!"){
         return Err(ParseError{content: ""})
@@ -257,7 +258,7 @@ pub(crate) fn lex_images<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Pars
     }
 }
 
-pub(crate) fn lex_links<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_links<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     if char_iter.peek() != Some(&"[") {
         return Err(ParseError{content: ""})
@@ -273,6 +274,7 @@ pub(crate) fn lex_links<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
         char_iter.next();
         let ref_id = title.strip_prefix("^").unwrap_or("");
         let mut note_text = String::new();
+        let note_index = char_iter.get_index();
         loop {
            note_text.push_str(char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or(""));
            char_iter.next();
@@ -297,7 +299,7 @@ pub(crate) fn lex_links<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
         if ref_id.contains(char::is_whitespace){
             return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")})
         }
-        return Ok(Token::Footnote(ref_id.to_string(), note_text.trim_start().to_string()));
+        return Ok(Token::Footnote(ref_id, char_iter.get_substring_from(note_index).unwrap_or("").trim()));
     }
     if char_iter.peek() != Some(&"(") {
         return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")})
@@ -309,17 +311,17 @@ pub(crate) fn lex_links<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
     }
     if char_iter.peek() == Some(&")") {
         char_iter.next();
-        return Ok(Token::Link(link.to_string(), Some(title.to_string()), None));
+        return Ok(Token::Link(link, Some(title), None));
     }
     if char_iter.peek() == Some(&" ") {
         let hover = char_iter.consume_while_case_holds(&|c| c != ")").unwrap_or("");
         char_iter.skip_while(|c| c != &"\n").next();
-        return Ok(Token::Link(link.to_string(), Some(title.to_string()), Some(hover.to_string())));
+        return Ok(Token::Link(link, Some(title), Some(hover)));
     }
     Err(ParseError{content: ""})
 }
 
-pub(crate) fn lex_side_carrot<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_side_carrot<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     match char_iter.peek() {
         Some("<") => {
             char_iter.next();
@@ -327,7 +329,7 @@ pub(crate) fn lex_side_carrot<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token,
             match char_iter.peek(){
                 Some(">") if s != "details" => {
                     char_iter.next();
-                    return Ok(Token::Link(s.to_string(), None, None))
+                    return Ok(Token::Link(s, None, None))
                 },
                 Some(">") if s == "details" => {
                     char_iter.next();
@@ -346,7 +348,7 @@ pub(crate) fn lex_side_carrot<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token,
     }
 }
 
-pub(crate) fn lex_plus_minus<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_plus_minus<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let s = char_iter.consume_while_case_holds(&|c| c == "-" || c == "+").unwrap_or("");
     match s.len() {
@@ -357,36 +359,36 @@ pub(crate) fn lex_plus_minus<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token,
     }
     let line = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
     if line.starts_with(" [ ] ") {
-        return Ok(Token::TaskListItem(TaskBox::Unchecked,line.strip_prefix(" [ ] ").unwrap_or("").to_string()))
+        return Ok(Token::TaskListItem(TaskBox::Unchecked,line.strip_prefix(" [ ] ").unwrap_or("")))
     } else if line.starts_with(" [x] ") {
-        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [x] ").unwrap_or("").to_string()))
+        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [x] ").unwrap_or("")))
     } else if line.starts_with(" [X] ") {
-        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [X] ").unwrap_or("").to_string()))
+        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [X] ").unwrap_or("")))
     } else if line.starts_with(" "){
-        return Ok(Token::UnorderedListEntry(line.strip_prefix(" ").unwrap_or("").to_string()))
+        return Ok(Token::UnorderedListEntry(line.strip_prefix(" ").unwrap_or("")))
     } else {
         return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")})
     }
 }
 
-pub(crate) fn lex_numbers<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_numbers<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let c = char_iter.next().unwrap();
     match char_iter.peek() {
         Some(".") => {
-            let dot = char_iter.next().unwrap();
+            char_iter.next().unwrap();
             if char_iter.peek() != Some(&" "){
                 return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")})
             }
             char_iter.next();
             let s = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
-            return Ok(Token::OrderedListEntry(s.to_string()))
+            return Ok(Token::OrderedListEntry(s))
         },
         _ => return Err(ParseError{content: c})
     }
 }
 
-pub(crate) fn lex_tilde<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_tilde<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let lead_tildes = match char_iter.consume_while_case_holds(&|s| s == "~"){
         Some(s) => s,
@@ -401,13 +403,13 @@ pub(crate) fn lex_tilde<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
                 return Err(ParseError{content: char_iter.get_substring_from(start_index).unwrap_or("")})
                 // return Err(ParseError{content: format!("{}{}{}",  lead_tildes, line, tail_tildes)})
             }
-            return Ok(Token::Strikethrough(line.to_string()));
+            return Ok(Token::Strikethrough(line));
         }
         _ => return Err(ParseError{content: lead_tildes}),
     }
 }
 
-fn parse_details<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>>{
+fn parse_details<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>>{
     let mut summary_line = char_iter.consume_while_case_holds(&|c| c != "\n").unwrap_or("");
     if summary_line.ends_with("\r") {
         summary_line = summary_line.strip_suffix("\r").unwrap_or("");
@@ -421,21 +423,23 @@ fn parse_details<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'
         0 => {return Err(ParseError{content: "<summary></summary>"})},
         _ => {},
     }
-    let mut remaining_text = char_iter.consume_until_tail_is("</details>").unwrap_or("").to_string();
+    let remaining_text_index = char_iter.get_index();
+    let mut remaining_text = char_iter.consume_until_tail_is("</details>").unwrap_or("");
     if remaining_text.contains("<details>") {
         let mut opens = remaining_text.matches("<details>").count();
         let mut closes = remaining_text.matches("</details>").count();
         while opens == closes {
-            remaining_text = remaining_text+char_iter.consume_until_tail_is("</details>").unwrap_or("");
+            char_iter.consume_until_tail_is("</details>").unwrap_or("");
+            remaining_text = char_iter.get_substring_from(remaining_text_index).unwrap_or("");
             opens = remaining_text.matches("<details>").count();
             closes = remaining_text.matches("</details>").count();
         }
     }
     let inner_tokens = crate::lex(remaining_text.strip_suffix("</details>").unwrap_or(""));
-    Ok(Token::Detail(summary_line.to_string(), inner_tokens))
+    Ok(Token::Detail(summary_line, inner_tokens))
 }
 
-pub(crate) fn lex_pipes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, ParseError<'a>> {
+pub(crate) fn lex_pipes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token<'a>, ParseError<'a>> {
     let start_index = char_iter.get_index();
     let mut lines = Vec::new();
     while char_iter.next_if_eq(&"|") == Some("|") {
@@ -450,13 +454,13 @@ pub(crate) fn lex_pipes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
     }
     let headings: Vec<_> = lines.remove(0).split("|")
         .filter(|&x| x != "")
-        .map(|x| x.trim().to_string())
+        .map(|x| x.trim())
         .collect();
     let alignments: Vec<_> = lines.remove(0).split("|")
         .filter(|&x| x != "")
         .map(|x| 
             {
-            match (x.trim().to_string().starts_with(":"), x.trim().to_string().ends_with(":")) {
+            match (x.trim().starts_with(":"), x.trim().ends_with(":")) {
                 (true, false) => Alignment::Left,
                 (true, true) => Alignment::Center,
                 (false, true) => Alignment::Right,
@@ -465,10 +469,10 @@ pub(crate) fn lex_pipes<'a>(char_iter: &mut MiniIter<'a>) -> Result<Token, Parse
         )
         .collect();
     let mut rows = Vec::new();
-    for l in lines.iter() {
-        let elements: Vec<String> = l.split("|")
+    for l in lines.into_iter() {
+        let elements: Vec<&str> = l.split("|")
         .filter(|&x| x != "")
-        .map(|x| x.trim().to_string())
+        .map(|x| x.trim())
         .collect();
         let mut r = Vec::new();
         for e in elements.into_iter() {
